    /*Project of game for amstad cpc 464.
    Copyright (C) 2013  Crappy Games(Antiono Lopez Quinto,David Mass Arroyo,Javier I. Cano Alberola,Jaume Gasa Gmez) 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

#include <stdlib.h>
#include <stdio.h>

#include "animation.h"
#include "screen.h"
#include "time.h"
#include "sprite.h"

// =======================================================================
//	CreateAnimation
// -----------------------------------------------------------------------
//	Creates and initialize an animation with the given parameters.
// =======================================================================
Animation* CreateAnimation(UI8 frameTime, UI8 maxFrames, UI8 frameWidth, UI8 frameHeight)
{
	Animation *animation = (Animation*) malloc(sizeof(Animation));
	
	if(animation == NULL)
		return NULL;
	
	animation->frame 			= 0;
	animation->frameTime 		= frameTime;
	animation->maxFrames 		= maxFrames;
	animation->frameWidth 	= frameWidth;
	animation->frameHeight 	= frameHeight;
	animation->timer 			= 0;
	animation->status 		= AS_STOPPED;
	
	return animation;
}

// =======================================================================
//	DeleteAnimation
// -----------------------------------------------------------------------
//	Frees the memory used by the animation.
// =======================================================================
void DeleteAnimation(Animation* animation)
{
	free(animation);
}

// =======================================================================
//	PlayAnimation
// -----------------------------------------------------------------------
//	Runs a stopped animation.
// =======================================================================
void PlayAnimation(Animation* animation)
{
	if(animation->status != AS_STOPPED)
		return;
	
	animation->frame 		= 0;
	animation->timer 		= GetTime();
	animation->status 	= AS_RUNNING;
}

// =======================================================================
//	PauseAnimation
// -----------------------------------------------------------------------
//	Pauses an running animation.
// =======================================================================
void PauseAnimation(Animation* animation)
{
	if(animation->status != AS_RUNNING)
		return;
	
	animation->status = AS_PAUSED;
}

// =======================================================================
//	ResumeAnimation
// -----------------------------------------------------------------------
//	Resumes a paused animation.
// =======================================================================
void ResumeAnimation(Animation* animation)
{
	if(animation->status != AS_PAUSED)
		return;

	animation->status = AS_RUNNING;
}

// =======================================================================
//	StopAnimation
// -----------------------------------------------------------------------
//	Stops a running or paused animation.
// =======================================================================
void StopAnimation(Animation* animation)
{
	if(animation->status != AS_RUNNING && animation->status != AS_PAUSED)
		return;
	
	animation->frame 		= 0;
	animation->timer 		= 0;
	animation->status 	= AS_STOPPED;
}

// =======================================================================
//	IsAnimationRunning
// -----------------------------------------------------------------------
//	Returns true if animation is running, false otherwise.
// =======================================================================
bool IsAnimationRunning(Animation* animation)
{
	return animation->status == AS_RUNNING;
}

// =======================================================================
//	IsAnimationPaused
// -----------------------------------------------------------------------
//	Returns true if animation is paused, false otherwise.
// =======================================================================
bool IsAnimationPaused(Animation* animation)
{
	return animation->status == AS_PAUSED;
}

// =======================================================================
//	IsAnimationStopped
// -----------------------------------------------------------------------
//	Returns true if animation is stopped, false otherwise.
// =======================================================================
bool IsAnimationStopped(Animation* animation)
{
	return animation->status == AS_STOPPED;
}

// =======================================================================
//	UpdateAnimation
// -----------------------------------------------------------------------
//	Updates a running animation, this function also renders it to the
//	screen.
// =======================================================================
void UpdateAnimation(Animation* animation)
{
	if(animation->status != AS_RUNNING)
		return;
	
	if(GetTime() - animation->timer >= animation->frameTime)
	{
		animation->frame = ++(animation->frame) % animation->maxFrames;
		animation->timer = GetTime();
		
		//DrawSprite((UI8*)0xC0A0, animation->frameWidth, animation->frameHeight, gPlayerSprite + K_PLAYER_WIDTH * K_PLAYER_HEIGHT * animation->frame);
	}
}

// =======================================================================
//	PrintAnimation
// -----------------------------------------------------------------------
//	Prints the animation data in the screen.
// =======================================================================
void PrintAnimation(UI8 x, UI8 y, Animation *animation, bool clearScreen)
{
	if(clearScreen)
		ClearScreen(K_DEFAULT_CLEAR_SCREEN_COLOR);
	
	SetCursor(x, y);
	
	printf("AnimAddr:\t0x%x\r\n", animation);
	printf("Frame:\t%u\r\n", animation->frame);
	printf("FrameTime:\t%u\r\n", animation->frameTime);
	printf("MaxFrames:\t%u\r\n", animation->maxFrames);
	printf("FrameWidth:\t%u\r\n", animation->frameWidth);
	printf("FrameHeight:\t%u\r\n", animation->frameHeight);
	printf("Timer:\t%u\r\n", animation->timer);
	
	switch(animation->status)
	{
		case AS_STOPPED	: printf("Status:\tSTOPPED\r\n"); break;
		case AS_PAUSED	: printf("Status:\tPAUSED\r\n"); break;
		case AS_RUNNING	: printf("Status:\tRUNNING\r\n"); break;
		default : break;
	}
}
