    /*Project of game for amstad cpc 464.
    Copyright (C) 2013  Crappy Games(Antiono Lopez Quinto,David Mass Arroyo,Javier I. Cano Alberola,Jaume Gasa Gmez) 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
 
#ifndef __GLOBALS_H__
#define __GLOBALS_H__

// =======================================================================
//							DEFINES
// =======================================================================

// BOOLEAN
#define bool unsigned char
#define true 1
#define false 0

// SCREEN
#define K_SCR_WIDTH 160
#define K_SCR_HEIGHT 200
#define K_SCR_WIDTH_MODE0 80

#define K_SCR_MODE 0
#define K_SCR_BORDER_COLOR 0x0
#define K_NUMBER_OF_COLORS 16

#define K_DEFAULT_CLEAR_SCREEN_COLOR 0x3

// TIME
#define K_TIME_UNITS_PER_SECOND 300
#define K_TIME_UNITS_PER_FRAME 6

// SPRITE
#define K_PLAYER_WIDTH 4
#define K_PLAYER_HEIGHT 10

// ANIMATION
#define K_DEFAULT_ANIMATION_FRAMES 3
#define K_DEFAULT_ANIMATION_FRAME_TIME 50

// MOTION
#define K_ACTOR_MOVEMENT_ENTRIES 2		// TESTING !!!
#define K_ACTOR_MAX_MOTION_PATHS 2		// TESTING !!!
#define K_DEFAULT_MOTION_PATH_SIZE 5

// MAP
#define K_TILE_WIDTH 8
#define K_TILE_HEIGHT 10

// =======================================================================
// 					TYPEDEFS, ENUMS AND STRUCTS
// =======================================================================

// ATOMIC DATA TYPES
typedef char 		I8;		// Range: [-128, 127]
typedef int		 	I16;		// Range: [-32768, 32767]
typedef unsigned char UI8;		// Range: [0, 255]
typedef unsigned int 	UI16;	// Range: [0, 65535]

/////////////////////////////////////////////////// POSITION AND DIRECTION
typedef struct
{
	UI8 x;
	UI8 y;

}Vector2D;

typedef enum
{
	DIR_DOWN		= 0,
	DIR_RIGHT 	= 1,
	DIR_LEFT 	= 2,
	DIR_UP 		= 3

}Direction;

//////////////////////////////////////////////////////////////// ANIMATION
typedef enum
{
	AS_STOPPED,
	AS_PAUSED,
	AS_RUNNING

}AnimationStatus;

typedef struct
{
	UI8  frame;				// Current frame
	UI8  frameTime;			// Time units that consumes one frame
	UI8  maxFrames;			// Max frames in one animation
	UI8  frameWidth;			// Width of one animation frame
	UI8  frameHeight;			// Height of one animation frame
	UI16 timer;				// Internal animation timer
	AnimationStatus status;	// Status of the animation

}Animation;

/////////////////////////////////////////////////////////////////// MOTION
typedef struct
{
	UI8	 x;				// X position of the pixel to reach
	UI8 	 y;				// Y position of the pixel to reach
	bool active;			// Is the actor movement active?
	bool repeat;			// Should the movement repeat forever? (At least until some external code cancels it)
	bool animateOnStop;	// Should the actor's animation continue when the location is reached?
	Direction direction;	// The direction of the movement

}Movement;

typedef struct
{
	UI8  			front;
	UI8 				count;
	Movement			*movements;

}MotionPath;

//////////////////////////////////////////////////////////////////// ACTOR
typedef enum
{
	AT_PLAYER,
	AT_ENEMY,
	AT_ITEM

}ActorType;

typedef struct
{
	UI8 			id;				// The unique actor identifier
	Vector2D 	tileCoords;		// Position of the actor in tile coordinates
	Vector2D 	screenCoords;		// Position of the actor in screen coordinates
	Vector2D 	velocity;		// Velocity components of the actor (px/s.)
	Movement		movement;		// Holds the current actor movement
	MotionPath	motionPath;		// Holds the programmed actor's movements
	UI8			width;			// Width of the actor
	UI8			height;			// Height of the actor
	Direction 	direction;		// Holds the direction faced by the actor
	const I8*	spriteData;		// Pointer to the bytes that represents an actor
	const I8*	sprite;			// Pointer to the logic actor's drawable bytes
	bool			visible;			// Determines if the actor is visible on the screen
	UI16			timer;			// Internal actor timer
	ActorType	type;			// Type of actor. An actor doesn't need to be a human :S
	Animation*	animation;		// Animation associated with the actor

}Actor;

// =======================================================================
// 							GLOBAL VARIABLES
// =======================================================================

// PALETTES
extern const UI8 gGamePalette[K_NUMBER_OF_COLORS];

// SPRITES
extern const I8 gPlayerSprite[];

// TESTING ---- !!!
//extern Movement gActorMovements[K_ACTOR_MOVEMENT_ENTRIES];
//extern MotionPath* gActorMotionPaths[K_ACTOR_MAX_MOTION_PATHS];

#endif