    /*Project of game for amstad cpc 464.
    Copyright (C) 2013  Crappy Games(Antiono Lopez Quinto,David Mass Arroyo,Javier I. Cano Alberola,Jaume Gasa Gmez) 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
 
#include <stdlib.h>
#include <stdio.h>

#include "motionpath.h"
#include "screen.h"
#include "input.h"

// =======================================================================
//	CreateMotionPath
// -----------------------------------------------------------------------
//	Creates an empty motion path.
// =======================================================================
void CreateMotionPath(MotionPath* motionPath, UI8 maxSize)
{
	Movement *movements = (Movement*) malloc(sizeof(Movement) * maxSize);
	
	if(movements == NULL)
	{
		printf("Not enough memory to create %u movements. ", maxSize);
		printf("Aborting the motion path creation.");
		return;
	}
	
	motionPath->front 	= 0;
	motionPath->count 	= 0;
	motionPath->movements = movements;
}

// =======================================================================
//	DeleteMotionPath
// -----------------------------------------------------------------------
//	Frees the memory used by a motion.
// =======================================================================
void DeleteMotionPath(MotionPath* motionPath)
{
	if(motionPath == NULL)
		return;
	
	motionPath->front		= 0;
	motionPath->count 	= 0;
	free(motionPath->movements);
	//free(motionPath);
}

// =======================================================================
//	NextMovement
// -----------------------------------------------------------------------
//	Grabs the next movement in the motion path.
// =======================================================================
Movement* GetNextMotionPathMovement(MotionPath* motionPath)
{
	Movement* next = NULL;
	
	if(IsMotionPathEmpty(motionPath))
		return NULL;
	
	next = &motionPath->movements[motionPath->front];
	
	motionPath->front++;
	motionPath->front %= 0xFF;
	motionPath->count--;
	
	return next;
}

// =======================================================================
//	AddMotionPathPixelMovement
// -----------------------------------------------------------------------
//	Adds a pixel movement to the motion path.
// =======================================================================
void AddMotionPathPixelMovement(UI8 x, UI8 y, Direction direction, MotionPath* motionPath)
{
	Movement* next = NULL;
	
	if(IsMotionPathFull(motionPath))
		return;
	
	next = &motionPath->movements[(motionPath->front + motionPath->count) % 0xFF];
	
	/*UI8 xPreviousMovement = 0;
	UI8 yPreviousMovement = 0;
	Movement* lastMovement = NULL;
	Movement* nextMovement = NULL;
	MotionPath* motionPath = &actor->motionPath;
	
	if(IsMotionPathFull(motionPath))
		return;
	
	// If the motion path is empty the first movement is relative to
	// the current actor screen position
	if(IsMotionPathEmpty(motionPath))
	{
		if(!actor->movement.active)
		{
			xPreviousMovement = actor->screenCoords.x;
			yPreviousMovement = actor->screenCoords.y;
		}
		else
		{
			xPreviousMovement = actor->movement.x;
			yPreviousMovement = actor->movement.y;
		}
	}
	else
	{
		lastMovement = &motionPath->movements[(motionPath->front + motionPath->count - 1) % 0xFF];
		xPreviousMovement = lastMovement->x;
		yPreviousMovement = lastMovement->y;
	}
	
	nextMovement = &motionPath->movements[(motionPath->front + motionPath->count) % 0xFF];
	
	switch(direction)
	{
		case DIR_DOWN :
			nextMovement->x = xPreviousMovement;
			nextMovement->y = yPreviousMovement + amount;
			break;
		case DIR_RIGHT :
			nextMovement->x = xPreviousMovement + amount;
			nextMovement->y = yPreviousMovement;
			break;
		case DIR_LEFT :
			nextMovement->x = xPreviousMovement - amount;
			nextMovement->y = yPreviousMovement;
			break;
		case DIR_UP :
			nextMovement->x = xPreviousMovement;
			nextMovement->y = yPreviousMovement - amount;
			break;
	}*/
	
	next->x				= x;
	next->y				= y;
	next->direction		= direction;
	next->active			= false;
	next->animateOnStop 	= false;
	motionPath->count++;
}

// =======================================================================
//	IsMotionPathFull
// -----------------------------------------------------------------------
//	Returns true if the motion path has reached it's maximum capcity,
//	false otherwise.
// =======================================================================
bool IsMotionPathFull(MotionPath* motionPath)
{
	return motionPath->count == 0xFF;
}

// =======================================================================
//	IsMotionPathEmpty
// -----------------------------------------------------------------------
//	Returns true if the motion path is empty, false otherwise.
// =======================================================================
bool IsMotionPathEmpty(MotionPath* motionPath)
{
	return motionPath->count == 0;
}

// =======================================================================
//	PrintMotionPath
// -----------------------------------------------------------------------
//	Prints the contents of a motion path.
// =======================================================================
void PrintMotionPath(UI8 x, UI8 y, MotionPath* motionPath, bool clearScreen)
{
	if(clearScreen)
		ClearScreen(K_DEFAULT_CLEAR_SCREEN_COLOR);
	
	SetCursor(x, y);
	
	printf("Addr:\t0x%x\r\n", motionPath);
	printf("Front:\t%u\r\n", motionPath->front);
	printf("Count:\t%u\r\n", motionPath->count);
}
