    /*Project of game for amstad cpc 464.
    Copyright (C) 2013  Crappy Games(Antiono Lopez Quinto,David Mass Arroyo,Javier I. Cano Alberola,Jaume Gasa Gmez) 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
 
#include <stdlib.h>

#include "screen.h"

// =======================================================================
//	SetMode
// -----------------------------------------------------------------------
//	Sets the video mode of the CPC.
// =======================================================================
void SetMode(UI8 mode)
{
	mode;
	
	__asm
		ld a, 4 (ix)
		call 0xBC0E	; SCR SET MODE
	__endasm;
}

// =======================================================================
//	SetBorderColor
// -----------------------------------------------------------------------
//	Sets the screen border color.
// =======================================================================
void SetBorderColor(UI8 color)
{
	color;
	
	__asm
		ld b, 4 (ix)
		ld c, b
		call #0xBC38	; SCR SET BORDER
	__endasm;
}

// =======================================================================
//	SetCursor
// -----------------------------------------------------------------------
//	Sets the cursor's vertical and horizontal position.
// =======================================================================
void SetCursor(UI8 x, UI8 y)
{
	x;
	y;
	
	__asm
		ld h, 4 (ix)
		ld l, 5 (ix)
		call #0xBB75	; TXT SET CURSOR
	__endasm;
}

// =======================================================================
//	SetPenColor
// -----------------------------------------------------------------------
//	Sets the color of the specified pen.
// =======================================================================
void SetPenColor(UI8 pen, UI8 color)
{
	pen;
	color;
	
	__asm
		ld a, 4 (ix)
		ld b, 5 (ix)
		ld c, b
		call #0xBC32	; SCR SET INK
	__endasm;
}

// =======================================================================
//	SetPalette
// -----------------------------------------------------------------------
//	Sets a palette of colors. Each color is assigned to a different pen.
// =======================================================================
void SetPalette(UI8 *palette, UI8 numberOfColors)
{
	UI8 i = 0;
	
	for(i; i < numberOfColors; ++i)
	{
		SetPenColor(i, palette[i]);
	}
}

// =======================================================================
//	GetLineAddress
// -----------------------------------------------------------------------
//	Returns the address of the given screen line number.
// =======================================================================
UI8* GetLineAddress(UI8 lineNumber)
{
	return (UI8*) 0xC000 + (lineNumber / 0x8) * 0x50 + (lineNumber % 0x8) * 0x800;
}

// =======================================================================
//	GetPixelAddress												[B]
// -----------------------------------------------------------------------
//	Returns the address of the pixel located at the x, y point of the
//	screen.
// =======================================================================
UI8* GetPixelAddress(UI8 x, UI8 y)
{
	return (UI8*) 0xC000 + (y / 0x8) * 0x50 + (y % 0x8) * 0x800 + (x / 0x2);
}

/*UI8* addr = NULL;
UI8* GetPixelAddress(UI8 x, UI8 y)
{
	x;
	y;
	
	__asm
		ld de, #0	; Last x line
		ld hl, #0	; First y line (inverted?)
		call 0xbc1d	; SCR DOT POSITION
		ld de, #_addr
		ld d, h
		ld e, l
	__endasm;
	
	return addr;
}*/

// =======================================================================
//	GetColorLPixMode0
// -----------------------------------------------------------------------
//	Returns the byte associated with the given color number for the
//	leftmost pixel.
// =======================================================================
UI8 GetColorLPixMode0(UI8 color)
{
	switch(color)
	{
		case 0 : return 0x0;
		case 1 : return 0x80;
		case 2 : return 0x08;
		case 3 : return 0x88;
		case 4 : return 0x20;
		case 5 : return 0xA0;
		case 6 : return 0x28;
		case 7 : return 0xA8;
		case 8 : return 0x02;
		case 9 : return 0x82;
		case 10 : return 0x0A;
		case 11 : return 0x8A;
		case 12 : return 0x22;
		case 13 : return 0xA2;
		case 14 : return 0x2A;
		case 15 : return 0xAA;
		default : break;
	}
	
	return 0x0;
}

// =======================================================================
//	GetColorRPixMode0
// -----------------------------------------------------------------------
//	Returns the byte associated with the given color number for the
//	rightmost pixel.
// =======================================================================
UI8 GetColorRPixMode0(UI8 color)
{
	switch(color)
	{
		case 0 : return 0x0;
		case 1 : return 0x40;
		case 2 : return 0x04;
		case 3 : return 0x44;
		case 4 : return 0x10;
		case 5 : return 0x50;
		case 6 : return 0x14;
		case 7 : return 0x54;
		case 8 : return 0x01;
		case 9 : return 0x41;
		case 10 : return 0x05;
		case 11 : return 0x45;
		case 12 : return 0x11;
		case 13 : return 0x51;
		case 14 : return 0x15;
		case 15 : return 0x55;
		default : break;
	}
	
	return 0x0;
}

// =======================================================================
//	GetColor2PixMode0
// -----------------------------------------------------------------------
//	Returns the byte associated with the given color number for 2 pixels.
//	Remember that two pixels in mode 0 equals to one byte.
// =======================================================================
UI8 GetColor2PixMode0(UI8 color)
{
	switch(color)
	{
		case 0 : return 0x0;
		case 1 : return 0xC0;
		case 2 : return 0x0C;
		case 3 : return 0xCC;
		case 4 : return 0x30;
		case 5 : return 0xF0;
		case 6 : return 0x3C;
		case 7 : return 0xFC;
		case 8 : return 0x03;
		case 9 : return 0xC3;
		case 10 : return 0x0F;
		case 11 : return 0xCF;
		case 12 : return 0x33;
		case 13 : return 0xF3;
		case 14 : return 0x3F;
		case 15 : return 0xFF;
		default : break;
	}
	
	return 0x0;
}

// =======================================================================
//	ClearScreen
// -----------------------------------------------------------------------
//	Clears the screen with the specified color.
// =======================================================================
void ClearScreen(UI8 color)
{
	UI8 i = 0;
	UI8 j = 0;
	UI8 mode0Color = mode0Color = GetColor2PixMode0(color);
	UI8* lineAddr = NULL;
	
	SetPenColor(0x0, mode0Color);
	
	__asm
		call 0xBC14	; SCR CLEAR
	__endasm;
}
