#include <stdio.h>

#include "time.h"
#include "screen.h"

// =======================================================================
//	GetTime
// -----------------------------------------------------------------------
//	Returns the time that has elapsed since the computer was switched on
//	or reset. The returned value units are 1/300 of a second.
//
//	EXAMPLE:
//		- Returned value 1 -> 1/300 * 1 =  3.33 ms. = 0.00333 s.
//		- Returned value 5 -> 1/300 * 5 = 16.66 ms. = 0.01666 s.
//		- Returned value 6 -> 1/300 * 6 = 20.00 ms. = 0.02000 s.
// =======================================================================
UI8 hTime, lTime;
UI16 GetTime()
{
	__asm
		call #0xBD0D	; KL TIME PLEASE
		push hl
		pop de
		ld hl, #_hTime
		ld (hl), d
		ld hl, #_lTime
		ld (hl), e
	__endasm;
	
	return (hTime << 8) + lTime;
}

// =======================================================================
//	StartGlobalTimers
// -----------------------------------------------------------------------
//	Starts the frame and FPS timers.
// =======================================================================
UI16 frameTimer, fpsTimer;
void StartGlobalTimers()
{
	frameTimer 	= GetTime();
	fpsTimer 	= GetTime();
}

// =======================================================================
//	FrameEnd
// -----------------------------------------------------------------------
//	This function should be called when the main loop reaches its end
//	because is responsible to stabilize the frame rate.
// =======================================================================
void FrameEnd()
{
	// If this frame has been rendered before the minimum amount of time
	// for one frame has elapsed, then wait until the minimum is reached
	if(GetTime() - frameTimer < K_TIME_UNITS_PER_FRAME)
	{
		while(GetTime() - frameTimer < K_TIME_UNITS_PER_FRAME) {}
	}
	// (Frame Rate Governing)
	// If this frame has been rendered after the minimum amount of time
	// for one frame has elapsed, then wait another frame
	else	
	{
		frameTimer = GetTime();
		while(GetTime() - frameTimer < K_TIME_UNITS_PER_FRAME) {}
	}
	
	FpsUpdate();
	frameTimer = GetTime();
}

// =======================================================================
//	FrameEnd
// -----------------------------------------------------------------------
//	This function should be called when the main loop reaches its end
//	because is responsible of 
// =======================================================================
UI16 fps;
void FpsUpdate()
{
	++fps;
	
	// Only update the FPS counter to the screen once each second
	if(GetTime() - fpsTimer >= K_TIME_UNITS_PER_SECOND)
	{
		SetCursor(1, 1);
		printf("%u FPS", fps);
		
		fpsTimer = GetTime();
		fps = 0;
	}
}
