    /*Project of game for amstad cpc 464.
    Copyright (C) 2013  Crappy Games(Antiono Lopez Quinto,David Massó Arroyo,Javier I. Cano Alberola,Jaume Gasa Gómez) 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

#include <stdlib.h>
#include <stdio.h>

#include "globals.h"
#include "actor.h"
#include "map.h"
#include "input.h"
#include "path.h"
#include "game.h"

Actor* CreateActor(unsigned char x, unsigned char y, const char *sprite, ActorType AT, ActorSubType ST)
{
	Actor *actorPtr = (Actor*)malloc(sizeof(Actor));
	
	if(actorPtr == NULL)
		return NULL;
	
	actorPtr->id = ACTOR_ID;
	actorPtr->x = x;
	actorPtr->y = y;
	actorPtr->xOld = x;
	actorPtr->yOld = y;
	actorPtr->sprite = sprite;
        actorPtr->AT_Actor = AT;
        actorPtr->ST_Actor = ST;
	
	AddActorToActorList(actorPtr, gActorList);
	
	ACTOR_ID++;
	
	return actorPtr;
}

void MoveActor(unsigned char x, unsigned char y, Actor *actorPtr)
{
	if(x < MAP_WIDTH && y < MAP_HEIGHT && gGameState.currentMap[x][y] == TT_None) // || IsItem(x,y)) //Si es un item se actualizan tambien los valores de deuda y suicidio
	{
		actorPtr->xOld = actorPtr->x;
		actorPtr->yOld = actorPtr->y;
		actorPtr->x = x;
		actorPtr->y = y;
                
                updateActor(x,y);
	}
}

void RandomActorPath(const Actor *actor, unsigned char steps)
{
	unsigned char i = 0;
	unsigned char xTmp = actor->x;
	unsigned char yTmp = actor->y;
	PathStack *pathStack = CreateActorPathStack(actor);
	
	for(i; i < steps; ++i)
	{
		xTmp += rand() % 3 + (-1);
		yTmp += rand() % 3 + (-1);
		PushPathNode(pathStack, xTmp, yTmp);
	}
}

void DrawActor(Actor *actorPtr)
{
	if(actorPtr->xOld <= -1 || actorPtr->yOld <= -1)
		return;
	
	DrawTileType(actorPtr->xOld, actorPtr->yOld, gGameState.currentMap[actorPtr->xOld][actorPtr->yOld]);
	DrawTile(actorPtr->x, actorPtr->y, actorPtr->sprite);
	
	actorPtr->xOld = -1;
	actorPtr->yOld = -1;
}

void DrawActors()
{
	ActorList *iterator = gActorList;
	
	while(iterator != NULL)
	{
		if(iterator->actor != NULL)
			DrawActor(iterator->actor);
		
		iterator = iterator->next;
	}
}

void AddActorToActorList(const Actor *actorPtr, ActorList *listPtr)
{
	ActorList *node = NULL;
	
	if(listPtr == NULL)
	{
		listPtr = CreateNode(listPtr);
		listPtr->actor = actorPtr;
		return;
	}
	
	node = CreateNode(listPtr);
	node->actor = actorPtr;
	node->next = NULL;
}

void RemoveActor(unsigned char actorId, ActorList *listPtr)
{
	ActorList *previousNode = NULL;
	ActorList *nextNode = NULL;
	
	if(listPtr == NULL || listPtr->actor == NULL)
		return;
	
	while(listPtr != NULL)
	{
		if(listPtr->actor->id == actorId)
		{
			nextNode = listPtr->next;
			
			// Remove actor from screen
			DrawTileType(listPtr->actor->x, listPtr->actor->y, gGameState.currentMap[listPtr->actor->x][listPtr->actor->y]);
			
			free(listPtr->actor);
			free(listPtr);
			
			if(previousNode != NULL)
				previousNode->next = nextNode;
			else
				listPtr = nextNode;
			
			return;
		}
		
		previousNode = listPtr;
		listPtr = listPtr->next;
	}
}

ActorList* CreateNode(ActorList *listPtr)
{
	if(listPtr == NULL)
	{
		listPtr = (ActorList*) malloc(sizeof(ActorList));
		listPtr->actor = NULL;
		listPtr->next = NULL;
		
		return listPtr;
	}
	
	while(listPtr->next != NULL)
	{
		listPtr = listPtr->next;
	}
	
	listPtr->next = (ActorList*) malloc(sizeof(ActorList));
	listPtr = listPtr->next;
	
	listPtr->actor = NULL;
	listPtr->next = NULL;
	
	return listPtr;
}

void RemoveActors()
{
	ActorList *listPtr = gActorList;
	ActorList *nextNode = NULL;
	
	while(listPtr != NULL)
	{
		nextNode = listPtr->next;
		
		DrawTileType(listPtr->actor->x, listPtr->actor->y, gGameState.currentMap[listPtr->actor->x][listPtr->actor->y]);
		free(listPtr->actor);
		free(listPtr);
		
		listPtr = nextNode;
	}
	
	gActorList = CreateNode(NULL);
}

PathStack* CreateActorPathStack(const Actor *actorPtr)
{
	unsigned char i = 0;
	PathStack *stackPtr = NULL;
	
	for(i; i < K_MAX_PATH_STACKS; ++i)
	{
		if(IsPathStackEmpty(&gActorPaths[i]))
		{
			stackPtr = &gActorPaths[i];
			DeletePathStack(stackPtr);
			break;
		}
	}
	
	if(stackPtr == NULL)
	{
		printf("All path stacks are full");
		return NULL;
	}
	
	CreatePathStack(stackPtr, actorPtr, K_DEFAULT_STACK_SIZE);
	
	return stackPtr;
}

void UpdateActorPaths()
{
	unsigned char i = 0;
	PathNode *node = NULL;
	PathStack *stackPtr = NULL;
	
	for(i; i < K_MAX_PATH_STACKS; ++i)
	{
		if(!IsPathStackEmpty(stackPtr = &gActorPaths[i]))
		{
			node = PopPathNode(stackPtr);
			MoveActor(node->x, node->y, stackPtr->actor);
		}
	}
}
 
void updateActor(unsigned char x, unsigned char y)
{
    unsigned int money = 0;
    
    Actor* item = gGameState.currentMapActors[x][y];
    
    if (item->AT_Actor == AT_ITEM)
    {
        switch (item->ST_Actor)
        {
            case ST_Money1:
                money = 200;
                break;
            case ST_Money2:
                money = 500;
                break;
            case ST_Money3:
                money = 1000;
                break;
        }
    }
     
    if (money > 0)
    {
        gGameState.debt -= money;
        gGameState.suicide += money;
        
        updateDebt();
        updateSuicide();
    }

}
