    /*Project of game for amstad cpc 464.
    Copyright (C) 2013  Crappy Games(Antiono Lopez Quinto,David Massó Arroyo,Javier I. Cano Alberola,Jaume Gasa Gómez) 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

#ifndef __GLOBALS_H__
#define __GLOBALS_H__

// ========================================================================
// 							DEFINES
// ========================================================================

#define K_SCR_WIDTH 160u			// Screen width
#define K_SCR_HEIGHT 200u			// Screen height
#define K_SCR_PPB 2u				// Pixels per byte

#define TILE_WIDTH 8
#define TILE_HEIGHT 10

#define MAP_WIDTH 20
#define MAP_HEIGHT 20

#define MAP_DATA_A_HEIGHT 12
#define MAP_DATA_B_HEIGHT 8

#define NUM_COLORS 16

#define K_FPS 50
#define K_TIME_UNITS_PER_FRAME 30	// Each unit = 1 / 300

#define K_MAX_PATH_STACKS 12
#define K_DEFAULT_STACK_SIZE 30

#define bool unsigned char
#define true 1
#define false 0

#define DEBTHOLDER_POS 0xC800
#define TIMEHOLDER_POS DEBTHOLDER_POS + SPRITE_NUMBER_WIDTH*6+10
#define SUICIDEHOLDER_POS TIMEHOLDER_POS + SPRITE_NUMBER_WIDTH*5+10

#define SPRITE_NUMBER_WIDTH 3
#define SPRITE_NUMBER_HEIGHT 6

// ========================================================================
// 					TYPEDEFS, ENUMS AND STRUCTS
// ========================================================================


typedef struct
{
    unsigned int mins;
    unsigned int secs;
    unsigned int lastTime;
}Time;

// ---> Maps
typedef enum
{
    TT_None,
    TT_Block,
    TT_Lava
}gTileType;

typedef enum
{
    AT_PLAYER,
    AT_ENEMY,
    AT_ITEM
}ActorType;

typedef enum
{
    ST_None     = 0,
    ST_Beggar1  = 1,
    ST_Beggar2  = 2,
    ST_Beggar3  = 3,
    ST_Money1   = 4,
    ST_Money2   = 5,
    ST_Money3   = 6
}ActorSubType;


typedef gTileType Map[MAP_WIDTH][MAP_HEIGHT];

// ---> Actors
typedef struct
{
	unsigned char id;
	unsigned char x;
	unsigned char y;
	int xOld;
	int yOld;
	const char *sprite;
        ActorType AT_Actor;
        ActorSubType ST_Actor;
}Actor;

struct ActorNode
{
	const Actor *actor;
	struct ActorNode *next;
};

typedef struct ActorNode ActorList;

// ---> Paths
typedef struct
{
	unsigned char x;
	unsigned char y;

}PathNode;

typedef struct
{
	int top;
	unsigned char maxSize;
	const Actor *actor;
	PathNode *contents;

}PathStack;

typedef Actor* Map_Actors[MAP_WIDTH][MAP_HEIGHT];
// ---> GameState
typedef struct
{
	Map currentMap;		// Current loaded map
        Map_Actors currentMapActors;
        Time time;
        unsigned int debt;
        unsigned int suicide;
}GameState;

// ========================================================================
// 							GLOBAL VARIABLES
// ========================================================================

extern ActorList *gActorList;
extern PathStack gActorPaths[K_MAX_PATH_STACKS];
extern GameState gGameState;

extern const gTileType map1[];
extern const ActorSubType actors_map1[];

extern const unsigned char GamePalette[NUM_COLORS];
extern const char BlockSprite[];
extern const char GroundSprite[];
extern const char LavaSprite[];
extern const char CharacterSprite[];
extern const char OptionsScreen[];
extern const char InitScreen[];
extern const char Money1Sprite[];
extern const char Money2Sprite[];
extern const char Money3Sprite[];
extern const char Enemy1Sprite[];
extern const char NumbersSprite[10][SPRITE_NUMBER_WIDTH * SPRITE_NUMBER_HEIGHT];

#endif